/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
q3master@hendriklipka.de
*/

package de.hendriklipka.q3master;

import java.util.*;
import java.io.*;

import org.apache.log4j.*;

public class Configuration
{
    private Category CAT=Category.getInstance(Configuration.class.getName());
    private Category LOG_CAT=Category.getInstance("q3master.config");

    private static Configuration _instance=new Configuration();

    private String _log4jConfigFile="cfg/q3master.lcf";
    private int _listenPort=27950;
    private int _serverTimeout=11*60*1000;
    private int _serverQueryTimeout=2000;
    private int _serverQueryTries=3;
    private boolean _filterProtocol=true;
    private boolean _filterFull=true;
    private boolean _filterEmpty=true;

    private String[] _broadcastAddresses=null;
    protected int _broadcastInterval=2*60*1000;

    protected String[] _broadcastMaster=null;
    
    protected Map _serverQueryHandlers=new HashMap();

    protected Map _heartbeatHandlers=new HashMap();

    private static final String PROPNAME_HANDLER_HEARTBEAT="handler.heartbeat.";
    private static final String PROPNAME_HANDLER_SERVERQUERY="handler.protocol.";

    public int getListenPort()
    {
        return _listenPort;
    }

    public int getServerTimeout()
    {
        return _serverTimeout;
    }

    public int getServerQueryTimeout()
    {
        return _serverQueryTimeout;
    }

    public int getServerQueryTries()
    {
        return _serverQueryTries;
    }

    public String getLog4jConfigFile()
    {
        return _log4jConfigFile;
    }

    public boolean filterProtocol()
    {
        return _filterProtocol;
    }

    public boolean filterFull()
    {
        return _filterFull;
    }

    public boolean filterEmpty()
    {
        return _filterEmpty;
    }

    public String[] getBroadcastAddresses()
    {
            return _broadcastAddresses;
    }

    public int getBroadcastInterval()
    {
        return _broadcastInterval;
    }

    public String[] getBroadcastMaster()
    {
            return _broadcastMaster;
    }

    public static Configuration getInstance()
    {
        return _instance;
    }
    
    public Map getServerQueryHandlers()
    {
        return _serverQueryHandlers;
    }

    public Map getHeartbeatHandlers()
    {
        return _heartbeatHandlers;
    }

    public void readConfigFile(String fName)
    {
        try
        {
            Properties props=new Properties();
            props.load(new FileInputStream(fName));
            if (props.containsKey("server.timeout"))
            {
                _serverTimeout=getIntValue(props,"server.timeout")*1000*60;
            }
            if (props.containsKey("serverquery.timeout"))
            {
                _serverQueryTimeout=getIntValue(props,"serverquery.timeout")*1000;
            }
            if (props.containsKey("server.broadcast.interval"))
            {
                _broadcastInterval=getIntValue(props,"server.broadcast.interval")*1000;
            }
            if (props.containsKey("server.broadcast.addresses"))
            {
                StringTokenizer st=new StringTokenizer(props.getProperty("server.broadcast.addresses")," ,;");
                Vector v=new Vector();
                while (st.hasMoreElements())
                {
                    String s=(String)st.nextElement();
                    v.addElement(s);
                }
                _broadcastAddresses=new String[v.size()];
                v.copyInto(_broadcastAddresses);
            }
            if (props.containsKey("register.master"))
            {
                StringTokenizer st=new StringTokenizer(props.getProperty("register.master")," ,;");
                Vector v=new Vector();
                while (st.hasMoreElements())
                {
                    String s=(String)st.nextElement();
                    v.addElement(s);
                }
                _broadcastMaster=new String[v.size()];
                v.copyInto(_broadcastMaster);
            }
            if (props.containsKey("serverquery.tries"))
            {
                _serverQueryTries=getIntValue(props,"serverquery.tries");
            }
            if (props.containsKey("listen.port"))
            {
                _listenPort=getIntValue(props,"listen.port");
            }
            if (props.containsKey("filter.protocol"))
            {
                _filterProtocol=getBooleanValue(props,"filter.protocol");
            }
            if (props.containsKey("filter.full"))
            {
                _filterFull=getBooleanValue(props,"filter.full");
            }
            if (props.containsKey("filter.empty"))
            {
                _filterEmpty=getBooleanValue(props,"filter.empty");
            }
            if (props.containsKey("log.log4j.configfile"))
            {
                _log4jConfigFile=props.getProperty("log.log4j.configfile");
            }
         
            Enumeration propNames=props.propertyNames();
            while (propNames.hasMoreElements())
            {
                String name=(String)propNames.nextElement();
                if (name.startsWith(PROPNAME_HANDLER_SERVERQUERY))
                {
                    String prot=name.substring(PROPNAME_HANDLER_SERVERQUERY.length());
                    String handler=props.getProperty(name);
                    _serverQueryHandlers.put(prot,handler);
                }
            }
            
            propNames=props.propertyNames();
            while (propNames.hasMoreElements())
            {
                String name=(String)propNames.nextElement();
                if (name.startsWith(PROPNAME_HANDLER_HEARTBEAT))
                {
                    String prot=name.substring(PROPNAME_HANDLER_HEARTBEAT.length());
                    String handler=props.getProperty(name);
                    _heartbeatHandlers.put(prot,handler);
                }
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    public void dumpConfig()
    {
        CAT.debug("server.timeout="+_serverTimeout);
        CAT.debug("serverquery.timeout="+_serverQueryTimeout);
        CAT.debug("serverquery.tries="+_serverQueryTries);
        CAT.debug("listen.port="+_listenPort);
        CAT.debug("filter.protocol="+_filterProtocol);
        CAT.debug("filter.full="+_filterFull);
        CAT.debug("filter.empty="+_filterEmpty);
        CAT.debug("log.log4j.configfile="+_log4jConfigFile);
        CAT.debug("server.bradcast.interval="+_broadcastInterval);
        CAT.debug("server.broadcast.addresses");
        if (null!=_broadcastAddresses)
        {
            for (int i=0;i<_broadcastAddresses.length;i++)
                CAT.debug("    address="+_broadcastAddresses[i]);
        }
        CAT.debug("register.master");

        if (null!=_broadcastMaster)
        {
            for (int i=0;i<_broadcastMaster.length;i++)
                CAT.debug("    address="+_broadcastMaster[i]);
        }
        
        if (null==_serverQueryHandlers||_serverQueryHandlers.isEmpty())
        {
            CAT.debug("no additional server query handlers configured");
        }
        else
        {
            CAT.debug("additional server query handlers");
            Iterator prots=_serverQueryHandlers.keySet().iterator();
            while (prots.hasNext())
            {
                String prot=(String)prots.next();
                String handler=(String)_serverQueryHandlers.get(prot);
                CAT.debug("  protocol="+prot+", class="+handler);
            }
        }
        
        if (null==_heartbeatHandlers||_heartbeatHandlers.isEmpty())
        {
            CAT.debug("no additional heartbeat handlers configured");
        }
        else
        {
            CAT.debug("additional heartbeat handlers");
            Iterator prots=_heartbeatHandlers.keySet().iterator();
            while (prots.hasNext())
            {
                String prot=(String)prots.next();
                String handler=(String)_heartbeatHandlers.get(prot);
                CAT.debug("  heartbeat type="+prot+", class="+handler);
            }
        }
    }

    protected boolean getBooleanValue(Properties props, String name)
    {
        String s=props.getProperty(name);
        if (null==s)
            return false;
        s=s.toLowerCase();
        if (s.equals("yes")||s.equals("1")||s.equals("true"))
            return true;
        return false;
    }

    protected int getIntValue(Properties props, String name)
    {
        try
        {
            String s=props.getProperty(name);
            s=s.trim();
            return Integer.parseInt(s);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            LOG_CAT.error(ex);
        }
        return -1;
    }
}